#!/bin/bash 
# BitTorrent download wrapper 0.1
# Copyright (C) 2005, Olivier Mehani <shtrom-btw@ssji.net>
#
# A handy wrapper around btdownloadcurses.sh to call from your favorite web
# browser, or just by hand, a term will pop up if needed.
#
# usage: btwrapper.sh TORRENTFILE
#
# With each call to this script, windows are added to one single SCREEN (created,
# detached, with the first call), allowing to see every download in a single
# terminal and to go on downloading without the need to keep an interactive
# session open on the system.
#
# This script allows the user to choose where to download the selected torrent.
# The locations are customizable for each user using the $HOME/btwrapperrc file.
# This file must have one directory per line. Lines starting which # or blank
# are ignored, as are lines containing non-existent directories. Example:
# /--------------------------------------------
# | # download distro ISOs here
# | /data/distros
# |
# | # the above line and this one are ignored
# | /one/more/download/directory
# |
#
# You can register torrent files to open with this script using the /etc/mailcap
# file. You can add a line like the following:
# application/x-bittorrent; /usr/bin/btwrapper.sh "%s"
#
# This script depends mainly on SCREEN and dialog, and the usual GNU utilities.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

# BitTorrent download client script
CLIENT=/usr/bin/btdownloadcurses.py 
#CLIENT="vim"
# arguments to pass when calling the client
ARGS="--max_upload_rate 5 --max_uploads 4" 
#ARGS=

# default download directory
DLDIR=$HOME/download

# terminal emulator program
TERMBIN=/usr/X11R6/bin/xterm
#TERMBIN=/usr/bin/gnome-terminal
# arguments to pass to the terminal emulator
TERMARGS="-e"

CONF=$HOME/.btwrapperrc

SCREEN_SESSION_NAME=btwrapper

BACKTITLE="BitTorrent download wrapper"

DLTITLE="Download directory selection"
DLDESC="Please select where you want to download"
DLDESC2="(default is $DLDIR)"

MSGOKTITLE="Download started"
MSGOKDESC="Started download of torrent"
MSGOKDESC2="in directory"

if [ -z "$1" ]; then
	echo "usage: $0 TORRENTFILE" >&2
	exit 1
fi
TORRENTFILE=$1
TORRENTNAME=`basename ${TORRENTFILE/.torrent/}`

export PATH=/usr/local/bin:/usr/bin:/bin

# make a list of choices suitable for dialog from the user's configuation file
function make_dir_list()
{
	CHOICES=""
	while read DIR ; do
		if echo $DIR | grep -q "^\([[:space:]]\)*\(#.*\)\?$"; then
			# this is a blank line or a comment, let's do nothing
			# visible (couldn't think of a cleaner solution)
			echo -n ""
		elif [ -d $DIR ]; then
			CHOICES="$CHOICES $DIR directory"
		fi
	done < $CONF
	echo $CHOICES
}

# display a dialog asking the user where to download the selected torrent
function choose_dir()
{
	OUTFILE=$1
	CHOICES=`make_dir_list`

	dialog --backtitle "$BACKTITLE" --title "$DLTITLE" \
	--cancel-label Default \
	--menu "$DLDESC $TORRENTFILE $DLDESC2" 0 0 0 $CHOICES 2>$OUTFILE
	if [ $? != 0 -o -z "`cat $OUTFILE`" ]; then
		echo $DLDIR > $OUTFILE
	fi
}

# get the session-name of a potentially existing "download screen", excluding
# dead ones
function get_running_screen()
{
	screen -ls | grep $SCREEN_SESSION_NAME | grep -v Dead | awk '{print $1}'
}

# the main magic starts here
if tty -s ; then
	# running in a term
	if [ -r $CONF ]; then
		TMPFILE=`mktemp	/tmp/btwrapper.XXXXXX`
		choose_dir $TMPFILE
		DLDIR=`cat $TMPFILE`
		rm $TMPFILE
	fi
	cp "$@" $DLDIR
	cd $DLDIR
	export STY=`get_running_screen`
	if [ -z "$STY" ]; then
		# no previous running screen, create a new one
		export -n STY
		screen -d -m -t "$TORRENTNAME" -S $SCREEN_SESSION_NAME $CLIENT $ARGS "$@"
	else
		# create a new window in the running scrren ($STY is not empty)
		# workaroud screen's bug #10074: we can't create a window in a
		# detached screen, so lets reattach it if it's not
		if screen -ls | grep $STY | grep -q Detached ; then
			screen -dmS temp screen -R $STY
			export WASDETACHED=yes
			while screen -ls | grep $STY | grep -q Detached ; do
				sleep 1
				echo -n "."
			done
			echo " attached"

		fi
		screen -t "$TORRENTNAME" $CLIENT $ARGS "$@"
		if [ "$WASDETACHED" == "yes" ]; then
			screen -D $STY
		fi
	fi
	dialog --backtitle "$BACKTITLE" --title "$MSGOKTITLE" \
	--msgbox "$MSGOKDESC $TORRENTFILE $MSGOKDESC2 $DLDIR" 0 0
else 
	# not running in a term, launch one
	$TERMBIN $TERMARGS $0 "$@"
fi
